from django.conf import settings

REDLINK_BATCH_SIZE = getattr(settings, "REDLINK_BATCH_SIZE", 100)
REDLINK_DEFAULT_LANGUAGE = getattr(settings, "REDLINK_DEFAULT_LANGUAGE", "en")

REDLINK_APPLICATIONS = [settings.REDLINK_ANDROID_APP_ID, settings.REDLINK_IOS_APP_ID]



from django.contrib import admin
from django.urls import path
from rest_framework.routers import SimpleRouter

from appchance.sections import views

urlpatterns = [
    path("admin/", admin.site.urls),
]

router = SimpleRouter()
router.register("sections", views.SectionViewSet, basename="sections")

urlpatterns += router.urls



import pytest
from appchance.redlink.models import (
    DeviceNotificationSettings,
    RedlinkDevice,
)
from appchance.redlink.serializers import RedlinkDeviceSerializer
from appchance.redlink.tests.factories import RedlinkDeviceFactory


@pytest.mark.django_db
class TestRedlinkDeviceSerializer:
    def test_serialize(self):
        device: RedlinkDevice = RedlinkDeviceFactory()
        DeviceNotificationSettings.objects.create(device=device)
        serializer = RedlinkDeviceSerializer(device)
        assert serializer.data == {
            "id": str(device.id),
            "registration_id": device.registration_id,
            "user": device.user_id,
            "name": device.name,
            "active": device.active,
            "type": device.type,
            "created": device.created.strftime("%Y-%m-%d %H:%M:%S"),
            "updated": device.created.strftime("%Y-%m-%d %H:%M:%S"),
            "settings": {
                "id": device.settings.id,
                "marketing": device.settings.marketing,
                "transactional": device.settings.transactional,
            },
        }



class RedlinkError(Exception):
    pass


class RedlinkAuthorizationError(RedlinkError):
    """
    Authorization data is invalid
    """


class RedlinkNotFoundError(RedlinkError):
    """
    Request was successfully processed but no results were returned
    """


class RedlinkServerError(RedlinkError):
    """
    Request was not processed due to server error
    """



# Generated by Django 3.0 on 2020-07-29 12:47

import uuid

import appchance
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="RedlinkDevice",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        help_text="Unique device identifier",
                        primary_key=True,
                        serialize=False,
                        unique=True,
                        verbose_name="Device ID",
                    ),
                ),
                ("registration_id", models.TextField()),
                ("session_key", models.CharField(blank=True, db_index=True, max_length=40)),
                ("name", models.CharField(max_length=255, null=True, verbose_name="Name")),
                ("active", models.BooleanField(default=True)),
                ("type", models.CharField(choices=[("ios", "ios"), ("android", "android")], max_length=10)),
                ("created", models.DateTimeField(auto_now_add=True)),
                ("updated", models.DateTimeField(auto_now=True)),
                (
                    "user",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="devices",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=(appchance.redlink.models.RedlinkClientMixin, models.Model),
        ),
        migrations.CreateModel(
            name="NotificationSettings",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("marketing", models.BooleanField(default=True)),
                ("transactional", models.BooleanField(default=True)),
                (
                    "device",
                    models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="notifications.RedlinkDevice"),
                ),
            ],
        ),
    ]



import swapper
from django.db import models
from django.utils.translation import gettext_lazy as _

from appchance.sections.querysets import BaseSectionQuerySet


class BaseSection(models.Model):
    POSITION_TOP = 1
    POSITION_CHOICES = [
        (POSITION_TOP, _("top")),
    ]

    TYPE_BANNER = 1
    TYPE_SLIDER = 2

    TYPE_CHOICES = [
        (TYPE_BANNER, _("banner")),
        (TYPE_SLIDER, _("slider")),
    ]

    name = models.CharField(blank=True, max_length=255)
    location = models.PositiveSmallIntegerField(_("location"), choices=POSITION_CHOICES, blank=True, null=True)
    type = models.PositiveSmallIntegerField(_("type"), choices=TYPE_CHOICES, blank=True, null=True)

    objects = BaseSectionQuerySet.as_manager()

    @property
    def content(self):
        """
        Prefetched content ordered by :class:`SectionContent` order

        :rtype: QuerySet(Content)
        """
        return self.content_list.order_by("order").select_related("content").all()

    class Meta:
        abstract = True


class Section(BaseSection):
    """
    May be swapped by setting PYCHANCE_SECTIONS_SECTION_MODEL
    """

    class Meta:
        swappable = swapper.swappable_setting("pychance_sections", "Section")


class BaseContent(models.Model):
    name = models.CharField(_("name"), blank=True, max_length=255)
    redirect_url = models.URLField(_("redirect url"), blank=True, null=True)
    content_url = models.URLField(_("content url"), blank=True, null=True)

    class Meta:
        abstract = True


class Content(BaseContent):
    class Meta:
        swappable = swapper.swappable_setting("pychance_sections", "Content")


class BaseSectionContent(models.Model):
    section = models.ForeignKey(
        swapper.get_model_name("pychance_sections", "Section"), on_delete=models.CASCADE, related_name="content_list"
    )
    content = models.ForeignKey(
        swapper.get_model_name("pychance_sections", "Content"), on_delete=models.CASCADE, related_name="content_list"
    )
    order = models.PositiveSmallIntegerField(_("order"), default=0)

    class Meta:
        abstract = True
        ordering = ("order",)


class SectionContent(BaseSectionContent):
    class Meta:
        swappable = swapper.swappable_setting("pychance_sections", "SectionContent")



import json
from collections import defaultdict
from datetime import datetime
from enum import Enum
from typing import Dict, List, Optional, TypedDict, Union

import requests
from appchance.redlink.errors import (
    RedlinkAuthorizationError,
    RedlinkNotFoundError,
    RedlinkServerError,
)


class ActionType(Enum):
    NONE = 1
    BROWSER = 2
    WEBVIEW = 3
    DEEPLINK = 4


class ReceiverType(Enum):
    DEVICE_RECEIVER = 1
    EMAIL_RECEIVER = 2
    NUMBER_RECEIVER = 3


class ReceiverDict(TypedDict, total=False):
    receiver: str
    external_id: Optional[str]
    type: ReceiverType


class RedlinkClient:
    def __init__(self, api_key):
        self.api_key = api_key

    DEFAULT_LANGUAGE = "en"
    URL = "https://api.redlink.pl/v1/push"
    DATE_FORMAT = "%Y-%m-%d %H:%M:%S"

    def send_push(
        self,
        applications: List[str],
        title: Union[str, Dict],
        body: Union[str, Dict],
        to: List[ReceiverDict],
        default_language: Optional[str] = None,
        silent: Optional[bool] = None,
        sound: Optional[str] = None,
        schedule_time: Optional[str] = None,
        ttl: Optional[int] = None,
        external_data: Optional[Dict] = None,
        subtitle: Optional[str] = None,
        lockscreen_visibility: Optional[int] = None,
        icon_small: Optional[str] = None,
        icon_large: Optional[str] = None,
        action_url: Optional[str] = None,
        action_type: Optional[int] = None,
        action_buttons: Optional[List[Dict]] = None,
    ):
        payload = self._parse_payload(
            applications=applications,
            to=to,
            title=title,
            body=body,
            default_language=default_language,
            silent=silent,
            sound=sound,
            schedule_time=schedule_time,
            ttl=ttl,
            subtitle=subtitle,
            lockscreen_visibility=lockscreen_visibility,
            external_data=external_data,
            icon_small=icon_small,
            icon_large=icon_large,
            action_url=action_url,
            action_type=action_type,
            action_buttons=action_buttons,
        )
        response = requests.post(url=self.URL, data=json.dumps(payload), headers=self.headers)
        return self._parse_response(response)

    @property
    def headers(self):
        return {"Authorization": self.api_key, "content-type": "application/json"}

    def _parse_payload(
        self,
        applications: List[str],
        title: Union[str, Dict],
        body: Union[str, Dict],
        to: List[ReceiverDict],
        default_language: Optional[str] = None,
        image: Optional[str] = None,
        silent: Optional[bool] = None,
        sound: Optional[str] = None,
        schedule_time: Optional[str] = None,
        ttl: Optional[int] = None,
        external_data: Optional[Dict] = None,
        subtitle: Optional[str] = None,
        lockscreen_visibility: Optional[int] = None,
        icon_small: Optional[str] = None,
        icon_large: Optional[str] = None,
        action_url: Optional[str] = None,
        action_type: Optional[int] = None,
        action_buttons: Optional[List[Dict]] = None,
    ):
        if default_language is None:
            default_language = self.DEFAULT_LANGUAGE

        if isinstance(title, str):
            title = {default_language: title}

        if isinstance(body, str):
            body = {default_language: body}

        payload: Dict = dict(
            applications=applications,
            to=dict(),
            title=title,
            body=body,
            defaultLanguage=default_language,
            scheduleTime=datetime.now().strftime(self.DATE_FORMAT),
            externalData=dict(),
            action=dict(type=ActionType.NONE.value, url=""),
        )

        for receiver in to:
            receiver["type"] = receiver["type"].value
        payload["to"] = to

        if image:
            payload["image"] = image
        if silent is not None:
            payload["silent"] = silent
        if sound:
            payload["sound"] = sound

        if schedule_time:
            payload["scheduleTime"] = schedule_time
        else:
            payload["scheduleTime"] = datetime.now().strftime(self.DATE_FORMAT)

        if ttl:
            payload["ttl"] = ttl
        if external_data:
            payload["externalData"] = external_data

        advanced: Dict = defaultdict(dict)
        if subtitle:
            advanced["subtitle"] = subtitle
        if lockscreen_visibility:
            advanced["lockscreenVisibility"] = lockscreen_visibility
        if icon_small:
            advanced["icon"]["small"] = icon_small
        if icon_large:
            advanced["icon"]["large"] = icon_large
        if advanced:
            payload["advanced"] = advanced
        if action_url:
            payload["action"].update({"url": action_url})
        if action_type:
            payload["action"].update({"type": action_type})

        if action_buttons:
            payload["actionButtons"] = action_buttons
        return payload

    def _parse_response(self, response):
        if response.status_code < 400:
            return response
        if response.status_code == 400:
            raise RedlinkAuthorizationError("Parse error.")
        if response.status_code == 401:
            raise RedlinkAuthorizationError("Invalid or missing token.")
        if response.status_code == 404:
            raise RedlinkNotFoundError("Receiver not found.")
        else:
            raise RedlinkServerError("Service unavailable.")



import factory
from appchance.redlink.models import (
    DeviceNotificationSettings,
    RedlinkDevice,
)
from factory import Faker


class RedlinkDeviceFactory(factory.DjangoModelFactory):
    name = Faker("pystr", max_chars=10)
    registration_id = Faker("pystr", max_chars=10)
    type = factory.Iterator(["android", "ios"])

    class Meta:
        model = RedlinkDevice


class DeviceNotificationSettingsFactory(factory.DjangoModelFactory):
    marketing = Faker("pybool")
    transactional = Faker("pybool")
    device = factory.SubFactory(RedlinkDeviceFactory)

    class Meta:
        model = DeviceNotificationSettings



from django.db import models


class PyShop(models.Model):
    """Shop."""

    name = models.CharField(max_length=120)
    code = models.CharField(max_length=50)

    class Meta:
        abstract = True

    def __str__(self):
        return self.name


class PyBrand(models.Model):
    """Product's brand."""

    name = models.CharField(max_length=80)

    class Meta:
        abstract = True

    def __str__(self):
        return self.name


class PyCategory(models.Model):
    """Category of products."""

    name = models.CharField(max_length=255)

    class Meta:
        abstract = True

    def __str__(self):
        return self.name


class PyProduct(models.Model):
    """Product in shop."""

    # General
    sku = models.CharField(max_length=20, unique=True)
    name = models.CharField(max_length=200)
    short_description = models.CharField(blank=True, max_length=255)
    description = models.TextField(blank=True)

    # Flags
    is_cc = models.BooleanField(default=False)
    is_active = models.BooleanField(default=True)
    is_deleted = models.BooleanField(default=False)

    class Meta:
        abstract = True

    def __str__(self):
        return f"({self.sku}) {self.name}"


class PyProductSize(models.Model):
    """Product size: width, height, depth and weight."""

    size_width = models.DecimalField(max_digits=20, decimal_places=6, default=0)
    size_height = models.DecimalField(max_digits=20, decimal_places=6, default=0)
    size_depth = models.DecimalField(max_digits=20, decimal_places=6, default=0)
    size_weight = models.DecimalField(max_digits=20, decimal_places=6, default=0)

    class Meta:
        abstract = True


class PyHomeSection(models.Model):
    """Homescreen section."""

    name = models.CharField(max_length=80)
    order = models.PositiveIntegerField(default=0)

    class Meta:
        abstract = True



def chunk_queryset(source_qs, chunk_size, query_log=None):
    """
    Yield pages of a queryset.
    """
    pk = None
    has_fields = hasattr(source_qs, "_fields") and source_qs._fields
    if has_fields:
        if "pk" not in source_qs._fields:
            raise Exception("The values() call must include the `pk` field")  # noqa

    field = source_qs.model._meta.pk
    # set the correct field name:
    # for ForeignKeys, we want to use `model_id` field, and not `model`,
    # to bypass default ordering on related model
    order_by_field = field.attname

    source_qs = source_qs.order_by(order_by_field)
    queryset = source_qs
    while True:
        if pk:
            queryset = source_qs.filter(pk__gt=pk)
        page = queryset[:chunk_size]
        if query_log is not None:
            query_log.write("{page.query}\n".format(page=page))
        page = list(page)
        nb_items = len(page)

        if nb_items == 0:
            return

        last_item = page[-1]
        # source_qs._fields exists *and* is not none when using "values()"
        if has_fields:
            pk = last_item["pk"]
        else:
            pk = last_item.pk

        yield page

        if nb_items < chunk_size:
            return



from appchance.redlink.models import (
    DeviceNotificationSettings,
    RedlinkDevice,
)
from django.contrib import admin


class DeviceNotificationSettingsInline(admin.StackedInline):
    model = DeviceNotificationSettings


class RedlinkDeviceAdminAbstract:
    list_display = (
        "id",
        "registration_id",
        "user",
        "name",
        "type",
        "created",
        "updated",
    )
    readonly_fields = ("user",)
    search_fields = ("id", "registration_id", "user__id")
    list_filter = ("settings__marketing", "settings__transactional")
    inlines = (DeviceNotificationSettingsInline,)


@admin.register(RedlinkDevice)
class RedlinkDeviceAdmin(RedlinkDeviceAdminAbstract, admin.ModelAdmin):
    pass



from django.conf import settings
from django.contrib.gis.db.models import PointField
from django.contrib.gis.geos import Point
from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from appchance.client.ftp import ClientSFTP


class PickupPointDPD(models.Model):
    ext_id = models.CharField(_("external id"), max_length=7)
    post_code = models.CharField(_("postal code"), max_length=6)
    address = models.CharField(_("address"), max_length=100)
    opening_hours = models.TextField(_("opening hours"))
    is_active = models.BooleanField(_("is active"), default=True)
    location = PointField(_("location"))
    additional_info = models.CharField(_("additional information"), max_length=250, blank=True)
    update_dt = models.DateTimeField(_("update"), null=True, blank=True)

    class Meta:
        abstract = True

    @staticmethod
    def fetch_pickup_points():
        sftp = ClientSFTP(
            settings.DPD_SFTP_HOST, settings.DPD_SFTP_PORT, settings.DPD_SFTP_USER, settings.DPD_SFTP_PASSWORD
        )
        sftp.get_latest(settings.DPD_SFTP_REMOTE_PATH, settings.DPD_SFTP_LOCAL_PATH)

    @classmethod
    def update_pickup_points(cls):
        with open(settings.DPD_SFTP_LOCAL_PATH, "r") as csv_file:
            next(csv_file)
            for line in csv_file:
                columns = line.split(";")
                if columns[0] != "\n" and columns[1][:2] in ["PL"]:
                    point, crtd = cls.objects.get_or_create(
                        ext_id=columns[1],
                        defaults={
                            "post_code": columns[2],
                            "address": "{} {}".format(columns[4], columns[7]),
                            "opening_hours": columns[10],
                            "location": Point([float(columns[14]), float(columns[13])]),
                            "additional_info": columns[15],
                        },
                    )
                    point.is_active = True if columns[11] == "Aktywny" else False
                    point.update_dt = timezone.now()
                    point.save(update_fields=["is_active"])

    @classmethod
    def fetch_and_update(cls):
        cls.fetch_pickup_points()
        cls.update_pickup_points()



from appchance.redlink.models import (
    DeviceNotificationSettings,
    RedlinkDevice,
)
from rest_framework.serializers import ModelSerializer


class DeviceNotificationSettingsSerializer(ModelSerializer):
    class Meta:
        model = DeviceNotificationSettings
        exclude = ("device",)
        extra_kwargs = {"id": {"read_only": True}}


class RedlinkDeviceSerializer(ModelSerializer):
    settings = DeviceNotificationSettingsSerializer(read_only=True)

    class Meta:
        model = RedlinkDevice
        fields = "__all__"
        extra_kwargs = {"id": {"read_only": True, "required": False}}

    def validate(self, attrs):
        self.Meta.model.objects.filter()
        return attrs



import factory
import swapper

Section = swapper.load_model("pychance_sections", "Section")
Content = swapper.load_model("pychance_sections", "Content")
SectionContent = swapper.load_model("pychance_sections", "SectionContent")


class SectionFactory(factory.django.DjangoModelFactory):
    location = factory.Iterator([choice[0] for choice in Section.POSITION_CHOICES])
    type = factory.Iterator([choice[0] for choice in Section.TYPE_CHOICES])
    name = factory.Faker("pystr")

    class Meta:
        model = Section


class ContentFactory(factory.django.DjangoModelFactory):
    name = factory.Faker("pystr")
    redirect_url = factory.Faker("ipv4")
    content_url = factory.Faker("ipv4")

    class Meta:
        model = Content


class SectionContentFactory(factory.django.DjangoModelFactory):
    order = factory.Faker("pyint")
    section = factory.SubFactory(SectionFactory)
    content = factory.SubFactory(ContentFactory)

    class Meta:
        model = SectionContent



from azure.storage.file import FileService
from azure.storage.file.models import FilePermissions
from django.conf import settings
from django.utils import timezone

read_permission = FilePermissions(read=True)


class ClientAFS:
    """Manage files on Azure File Storage.
    Default expiry time of SAS signature is set to one hour.
    """

    def __init__(self, account_name, account_key, expiry=3600):
        self.account_name = account_name
        self.client = FileService(account_name=account_name, account_key=account_key)
        self.expiry = expiry

    def get_file_sas(self, share_name, file_name):
        """Fetch token for temporary public file read access."""
        return self.client.generate_file_shared_access_signature(
            share_name=share_name,
            file_name=file_name,
            permission=read_permission,
            expiry=timezone.now() + timezone.timedelta(seconds=self.expiry),
        )

    def public_file_link(self, share_name, file_name):
        return (
            f"https://{self.account_name}.file.core.windows.net/"
            f"{file_name}?{self.get_file_sas(share_name, file_name)}"
        )



import pytest
import swapper

from appchance.sections.tests.factories import SectionContentFactory

Section = swapper.load_model("pychance_sections", "Section")
SectionContent = swapper.load_model("pychance_sections", "SectionContent")


@pytest.mark.django_db
class TestSectionQuerySet:
    def test_prefetch_content__content_prefetched(self):
        SectionContentFactory()
        sections = Section.objects.prefetch_content().all()
        assert len(sections[0]._prefetched_objects_cache) > 0



# Generated by Django 3.1.2 on 2020-10-15 15:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("sections_app", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="customcontent",
            name="content_url",
            field=models.URLField(blank=True, null=True, verbose_name="content url"),
        ),
        migrations.AddField(
            model_name="customcontent",
            name="img",
            field=models.URLField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="customcontent",
            name="name",
            field=models.CharField(blank=True, max_length=255, verbose_name="name"),
        ),
        migrations.AddField(
            model_name="customcontent",
            name="redirect_url",
            field=models.URLField(blank=True, null=True, verbose_name="redirect url"),
        ),
    ]



import setuptools

setuptools.setup(
    name="appchance-django",
    version="0.2.3",
    author_email="backend@appchance.com",
    short_description="Appchance Django Extensions",
    description="Common extensions for Django projects.",
    long_description=open("README.md", "r").read(),
    long_description_content_type="text/markdown",
    url="https://bitbucket.org/appchance/appchance-django/",
    packages=setuptools.find_packages(),
    install_requires=["django", "swapper"],
    scripts=[],
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "Operating System :: POSIX :: Linux",
    ],
    python_requires=">=3.8",
)



from unittest.mock import MagicMock, call

import pytest
from appchance.redlink import app_settings
from appchance.redlink.models import RedlinkDevice
from appchance.redlink.redlink_client import ActionType, ReceiverType
from appchance.redlink.tests.factories import RedlinkDeviceFactory
from django.conf import settings


@pytest.mark.django_db
class TestRedlinkDeviceQuerySet:
    def setup(self):
        app_settings.REDLINK_DEFAULT_LANGUAGE = "en"

    def test_send_push__single_device(self, settings):
        device = RedlinkDeviceFactory()
        queryset = RedlinkDevice.objects.get_queryset()
        queryset._redlink_client = MagicMock()
        queryset.send_push(
            title="Test push", body="Test body", external_data={"test1": "test data 1", "test2": "test data 2"},
        )
        queryset._redlink_client.send_push.assert_called_with(
            applications=[settings.REDLINK_ANDROID_APP_ID, settings.REDLINK_IOS_APP_ID],
            to=[{"receiver": device.registration_id, "type": ReceiverType.DEVICE_RECEIVER}],
            title={"en": "Test push"},
            body={"en": "Test body"},
            external_data={"test1": "test data 1", "test2": "test data 2"},
            action_type=ActionType.WEBVIEW.value,
            default_language="en",
        )

    def test_send_push__multiple_devices__chunk(self):
        device = sorted(RedlinkDeviceFactory.create_batch(3), key=lambda obj: obj.id)
        queryset = RedlinkDevice.objects.get_queryset()
        queryset._redlink_client = MagicMock()
        queryset.CHUNK_SIZE = 2
        queryset.send_push(
            title="Test push", body="Test body", external_data={"test1": "test data 1", "test2": "test data 2"},
        )
        queryset._redlink_client.send_push.assert_has_calls(
            calls=[
                call(
                    applications=[settings.REDLINK_ANDROID_APP_ID, settings.REDLINK_IOS_APP_ID],
                    to=[
                        {"receiver": device[0].registration_id, "type": ReceiverType.DEVICE_RECEIVER},
                        {"receiver": device[1].registration_id, "type": ReceiverType.DEVICE_RECEIVER},
                    ],
                    title={"en": "Test push"},
                    body={"en": "Test body"},
                    external_data={"test1": "test data 1", "test2": "test data 2"},
                    action_type=ActionType.WEBVIEW.value,
                    default_language="en",
                ),
                call(
                    applications=[settings.REDLINK_ANDROID_APP_ID, settings.REDLINK_IOS_APP_ID],
                    to=[{"receiver": device[2].registration_id, "type": ReceiverType.DEVICE_RECEIVER}],
                    title={"en": "Test push"},
                    body={"en": "Test body"},
                    external_data={"test1": "test data 1", "test2": "test data 2"},
                    action_type=ActionType.WEBVIEW.value,
                    default_language="en",
                ),
            ]
        )


@pytest.mark.django_db
class TestRedlinkDevice:
    def test_send_push(self):
        device: RedlinkDevice = RedlinkDeviceFactory()
        device._redlink_client = MagicMock(DEFAULT_LANGUAGE="en")

        device.send_push(
            title="Test push", body="Test body", external_data={"test1": "test data 1", "test2": "test data 2"},
        )
        device._redlink_client.send_push.assert_called_with(  # type: ignore
            applications=[settings.REDLINK_ANDROID_APP_ID, settings.REDLINK_IOS_APP_ID],
            to=[{"receiver": device.registration_id, "type": ReceiverType.DEVICE_RECEIVER}],
            title={"en": "Test push"},
            body={"en": "Test body"},
            external_data={"test1": "test data 1", "test2": "test data 2"},
            action_type=ActionType.WEBVIEW.value,
            default_language="en",
        )



from appchance.multi_db import db_config


class MultiDatabaseMiddlewareAbstract:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        db_config.db = self._get_db_name(request=request)

        response = self.get_response(request)

        if hasattr(db_config, "db"):
            del db_config.db

        return response

    class Meta:
        abstract = True

    def _get_db_name(self, request):
        """
        Implement this function and return target database name.

        :param request:
        :return:
        """
        raise NotImplementedError
